"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runCli = void 0;

var _path = _interopRequireDefault(require("path"));

var _readline = _interopRequireDefault(require("readline"));

var _types = require("@interaktiv/types");

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _constants = require("../constants");

var _config = require("../config");

var _runMiAppBuild = require("../core/runMiAppBuild");

const runCli = async (argv, logger = console) => {
  const config = await (0, _config.buildConfiguration)({
    argv
  });
  let skipExecution = false;

  try {
    skipExecution = await shouldSkip({
      argv,
      config,
      logger
    });
  } catch (err) {
    logger.warn(`${config.prefix}${err.message}, skipping plugin execution.\n`);
    skipExecution = true;
  }

  if (skipExecution) return {};

  try {
    await checkGitIgnore(config);
  } catch (_err) {
    // Ignore error just log a warning
    logger.warn(`${config.prefix}This plugin will overwrite your tiapp.xml. You should add "${_path.default.basename(config.tiappXml)}" to your .gitignore file.`);
  }

  logger.info(`${config.prefix}All checks done.`);
  return (0, _runMiAppBuild.runMiAppBuild)({
    config,
    logger
  });
};
/**
 * Checks if we should skip the plugin execution
 *
 * @param {Object} opts The options to check if we should skip
 * @param {Object} opts.argv The passed in argv from the cli
 * @param {Object} opts.config The build configuration
 * @param {Object} opts.logger The logger to output skip reasons
 * @return {Promise<boolean>} A promise that resolves with true if we should skip the plugin execution otherwise with false
 */


exports.runCli = runCli;

async function shouldSkip({
  argv,
  config,
  logger
}) {
  // Skip if Titanium Module detected
  if (config.isTitaniumModule) {
    logger.info(`${config.prefix}Titanium Module directory detected, skipping plugin execution.\n`);
    return true;
  } // Skip plugin if skip param given


  if (Object.prototype.hasOwnProperty.call(argv, 'skip-miapp-build')) {
    logger.info(`${config.prefix}Found '--skip-miapp-build', skipping plugin execution.\n`);
    return true;
  } // Skip if skip file found


  const skipPluginFileExists = await _fsExtra.default.exists(config.skipPlugin);

  if (skipPluginFileExists) {
    logger.info(`${config.prefix}Found '${_constants.SKIP_PLUGIN_FILE}' file, skipping plugin execution.\n`);
    return true;
  } // Skip if template file for tiapp.xml is not found


  if (config.isTemplateExistent === false) {
    logger.warn(`${config.prefix}Template file not found. This should be fine only if you don't want to use miapp-build in this project, skipping plugin execution.\n`);
    return true;
  } // Skip if plugin config is not found


  const miapprcExists = await _fsExtra.default.exists(config.miapprc);

  if (miapprcExists === false) {
    logger.warn(`${config.prefix}Missing '${config.miapprc}' file, skipping plugin execution.\n`);
    return true;
  } // Skip if cli selected sdk is another than the project used sdk. For this
  // case the plugin gets called a second time anyways


  const {
    sdk
  } = argv;
  const {
    titanium
  } = await (0, _runMiAppBuild.fetchMiAppRcSettings)({
    config
  });

  if ((0, _types.isString)(sdk) && sdk !== titanium.sdkVersion) {
    logger.info(`${config.prefix}Plugin executed with wrong Titanium SDK version, skipping plugin execution.\n`);
    return true;
  }

  return false;
}

function checkGitIgnore(config) {
  return new Promise((resolve, reject) => {
    let found = false;

    const lineReader = _readline.default.createInterface({
      input: _fsExtra.default.createReadStream(config.gitIgnore)
    });

    const searchRule = _path.default.basename(config.tiappXml);

    lineReader.on('line', line => {
      const trimmed = line.trim();

      if (trimmed === searchRule || trimmed === `/${searchRule}`) {
        found = true;
        lineReader.close();
      }
    });
    lineReader.on('close', () => {
      if (found) resolve();else reject(new Error(`${searchRule} not found in .gitignore`));
    });
  });
}