"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runMiAppBuild = runMiAppBuild;
exports.getCurrentBuildNumbers = exports.fetchMiAppRcSettings = exports.bumpBuildNumbers = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _path = _interopRequireDefault(require("path"));

var _moduleCopier = _interopRequireDefault(require("@titanium/module-copier"));

var _json = require("@interaktiv/json");

var _types = require("@interaktiv/types");

var _execa = _interopRequireDefault(require("execa"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _glob = _interopRequireDefault(require("glob"));

var _kleur = require("kleur");

var _lodash = _interopRequireDefault(require("lodash"));

var _plist = _interopRequireDefault(require("plist"));

var _walkObject = require("../lib/walk-object");

var _utils = require("../lib/utils");

var _diff = require("../lib/diff");

async function runMiAppBuild({
  config,
  logger
}) {
  logger.debug(`Platform: ${config.platform}`);
  logger.debug(`Target: ${config.target}`);
  logger.debug(`Deploy Type: ${config.deployType}`);
  logger.debug(`miappenv flag: ${config.miappenv || 'Not available'}`);
  logger.info(`${config.prefix}Generating build numbers.`);
  const buildNumbers = await getCurrentBuildNumbers({
    config
  });
  logger.info(`${config.prefix}Generating ${_path.default.basename(config.tiappXml)}.`);
  const settings = await fetchMiAppRcSettings({
    config
  });
  const {
    versionCode,
    cfBundleVersion
  } = bumpBuildNumbers(buildNumbers, settings);
  const {
    platform
  } = config;
  const debuggable = !config.ENV_PRODUCTION;
  const updatedSettings = (0, _extends2.default)({}, settings, {
    app: (0, _extends2.default)({}, settings.app, {
      // First match wins
      bundleIdentifier: [platform === 'ios' ? settings.ios.bundleIdentifier : null, platform === 'android' ? settings.android.packageName : null, settings.app.bundleIdentifier].filter(Boolean).shift()
    }),
    android: (0, _extends2.default)({}, settings.android, {
      debuggable,
      versionCode,
      versionName: `${settings.app.version} (${versionCode})`
    }),
    ios: (0, _extends2.default)({}, settings.ios, {
      buildNumber: cfBundleVersion,
      cfBundleVersion
    }),
    push: (0, _extends2.default)({}, settings.push, {
      // Check if Urban Airship in_production property is given
      // and set it accordingly to deploy type.
      inProduction: config.ENV_PRODUCTION
    })
  });
  const composed = await compose({
    config,
    settings: updatedSettings
  });
  Object.entries((0, _walkObject.walk)((0, _diff.diff)(settings, updatedSettings))).map(([key, value]) => `${config.prefix}Updated ${(0, _kleur.cyan)(key)} to: ${(0, _kleur.yellow)(value)}`).forEach(msg => logger.info(msg));
  logger.debug(`${config.prefix}tiapp.xml contents:\n\n${composed}\n\n`);
  logger.info(`${config.prefix}Successfully wrote ${config.tiappXml}.`);
  logger.info(`${config.prefix}Updating build numbers.`);
  await updateBuildNumbers({
    config,
    logger,
    settings: updatedSettings
  });
  logger.info(`${config.prefix}Copying npm modules.`); // WORKAROUND: For "no such file or directory, open '/Users/kbueschel/workspace-test/MyApp/build/widgets.json'", create empty file

  _fsExtra.default.ensureFileSync(_path.default.join(config.rootDir, 'build', 'widgets.json'));

  _moduleCopier.default.executeSync({
    projectPath: config.rootDir,
    targetPath: config.resources,
    includeOptional: true
  });

  logger.info(`${config.prefix}Done preparing your app for build.\n`);
  return config;
}

async function updateBuildNumbers({
  config,
  logger,
  settings
}) {
  const {
    app: {
      version
    },
    ios: {
      cfBundleVersion
    },
    android: {
      versionName
    }
  } = settings; // Update iOS root.plist app version default value

  const {
    settingsPlistFile
  } = config;

  if (await _fsExtra.default.exists(settingsPlistFile)) {
    const settingsPlist = _plist.default.parse(await _fsExtra.default.readFile(settingsPlistFile, 'utf-8'));

    const appVersionInSettings = `${version} (${cfBundleVersion})`;
    settingsPlist.PreferenceSpecifiers.reduce((acc, setting) => {
      if (setting.Key && setting.Key === 'appVersionSettings') {
        // eslint-disable-next-line no-param-reassign
        setting.DefaultValue = appVersionInSettings;
      }

      return acc;
    }, []);
    await _fsExtra.default.writeFile(settingsPlistFile, _plist.default.build(settingsPlist), 'utf-8');
    logger.info(`${config.prefix}Updated version in settings plist ${(0, _kleur.cyan)(settingsPlistFile)} to: ${(0, _kleur.yellow)(appVersionInSettings)}`);
  } // Update Android strings.xml


  return new Promise((resolve, reject) => {
    (0, _glob.default)(_path.default.join(config.i18n, '/**/strings.xml'), {
      pwd: config.rootDir
    }, async (globError, files) => {
      if (globError) {
        logger.error(globError);
        reject(new Error(globError));
        return;
      }

      await files.reduce(async (future, file) => {
        await future;

        try {
          const {
            changed
          } = await (0, _utils.replaceVersionInStringsFile)(file, versionName);
          logger.info(`${config.prefix}Updated version in ${(0, _kleur.cyan)(file)} to: ${(0, _kleur.yellow)(changed)}`);
        } catch (err) {
          logger.error(`${config.prefix}${err.message}, skipping`);
        }

        return Promise.resolve();
      }, Promise.resolve());
      resolve();
    });
  });
}

const bumpBuildNumbers = (buildNumbers, settings) => {
  const versionCode = parseInt(buildNumbers.versionCode || settings.android.versionCode, 10) + 1;
  const cfBundleVersion = parseInt(buildNumbers.cfBundleVersion || settings.ios.cfBundleVersion, 10) + 1;
  return {
    versionCode,
    cfBundleVersion
  };
};

exports.bumpBuildNumbers = bumpBuildNumbers;

const fetchMiAppRcSettings = async ({
  config
}) => {
  const {
    rootDir
  } = config;
  const {
    miapprc: miapprcFile,
    miappenv = _path.default.basename(rootDir).toLowerCase()
  } = config;
  const miapprc = (0, _json.parseJsonMap)(await _fsExtra.default.readFile(miapprcFile, 'utf-8'));
  const {
    settings
  } = miapprc.configs.find(_config => _config.name === miappenv) || {};

  if ((0, _types.isNil)(settings) || (0, _types.isEmpty)(settings)) {
    throw new Error(`Couldn't find the environment "${miappenv}" in file ${miapprcFile}.`);
  }

  return settings;
};

exports.fetchMiAppRcSettings = fetchMiAppRcSettings;

async function compose({
  config,
  settings
}) {
  const template = _lodash.default.template(await _fsExtra.default.readFile(config.tiappXmlTemplate, 'utf-8'));

  const composed = template(settings);
  await _fsExtra.default.outputFile(config.tiappXml, composed, 'utf-8');
  return composed;
}
/**
 * Checks requirements and generates build numbers
 *
 * @private
 * @param {Object} config - The config object
 * @returns {Promise} - A promise that resolves with an object containing the current and bumped ios and android app version numbers
 */


const getCurrentBuildNumbers = async ({
  config
}) => {
  const {
    rootDir
  } = config;
  const appOrVersionNotFoundRegExpressions = [/no (application|version) was found/gim, /forbidden/gim, /notFound/gim, /Package not found/gim];
  let currentBuildNumber = null;

  try {
    await (0, _utils.runBundleAction)('ios', ['fetch_app_version_numbers'], {
      cwd: rootDir,
      pipeStdout: true
    });
    const iosAppVersionJson = await _fsExtra.default.readFile(_path.default.join(rootDir, 'ios-app-version.json'), 'utf-8');
    const {
      result: {
        buildNumber = 0
      }
    } = (0, _json.parseJsonMap)(iosAppVersionJson);
    const parsedBuildNumber = parseInt(buildNumber, 10);
    currentBuildNumber = (0, _types.isNumber)(parsedBuildNumber) ? parsedBuildNumber : null;
  } catch (err) {
    // Ignore errors that dealing with app or version not found cause the app is
    // currently not uploaded to store or no app version is created. If the
    // provided Google Play Json Key is not for this account also ignore the
    // error. Otherwise bubble the error up
    const {
      message
    } = err;

    if (appOrVersionNotFoundRegExpressions.every(regex => regex.test(message) === false)) {
      throw err;
    }
  }

  let currentVersionCode = null;

  try {
    const googlePlayJsonKeyResult = (0, _json.parseJsonMap)((await (0, _execa.default)('mibuilder', ['config:get', 'android.googlePlayJsonKeyPath', '--no-banner', '--json'])).stdout);
    const googlePlayJsonKey = (0, _types.get)(googlePlayJsonKeyResult, 'result[0].value');
    await (0, _utils.runBundleAction)('android', ['fetch_app_version_numbers', `json_key:${googlePlayJsonKey}`], {
      cwd: rootDir,
      pipeStdout: true
    });
    const androidAppVersionJson = await _fsExtra.default.readFile(_path.default.join(rootDir, 'android-app-version.json'), 'utf-8');
    const {
      result: {
        versionCode = 0
      }
    } = (0, _json.parseJsonMap)(androidAppVersionJson);
    const parsedVersionCode = parseInt(versionCode, 10);
    currentVersionCode = (0, _types.isNumber)(parsedVersionCode) ? parsedVersionCode : null;
  } catch (err) {
    // Ignore errors that dealing with app or version not found cause the app is
    // currently not uploaded to store or no app version is created. If the
    // provided Google Play Json Key is not for this account also ignore the
    // error. Otherwise bubble the error up
    const {
      message
    } = err;

    if (appOrVersionNotFoundRegExpressions.every(regex => regex.test(message) === false)) {
      throw err;
    }
  }

  return {
    cfBundleVersion: currentBuildNumber,
    versionCode: currentVersionCode
  };
};

exports.getCurrentBuildNumbers = getCurrentBuildNumbers;