"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createErrorOutput = createErrorOutput;
exports.die = die;
exports.dieWithNode = dieWithNode;
exports.runBundleAction = runBundleAction;
exports.trim = exports.replaceVersionInStringsFile = exports.getAndValidateProjectPaths = exports.XML = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _path = _interopRequireDefault(require("path"));

var _errors = require("@interaktiv/errors");

var _execa = _interopRequireDefault(require("execa"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _lodash = _interopRequireDefault(require("lodash"));

var _xmldom = require("xmldom");

var _xpath = _interopRequireDefault(require("xpath"));

var CONSTANTS = _interopRequireWildcard(require("../constants"));

const XML = {
  getNodeText(node) {
    if (node == null) return '';
    const serializer = new _xmldom.XMLSerializer();
    let str = '';
    let c = 0;

    for (; c < node.childNodes.length;) {
      if (node.childNodes[c].nodeType === 3) {
        str += serializer.serializeToString(node.childNodes[c]);
      }

      c += 1;
    }

    return str.replace(/&amp;/g, '&');
  },

  getElementsFromNodes(nodeList) {
    const elems = [];

    if (nodeList && nodeList.length) {
      let i = 0;
      const l = nodeList.length;

      for (; i < l;) {
        const node = nodeList.item(i);

        if (node.nodeType === 1) {
          elems.push(node);
        }

        i += 1;
      }
    }

    return elems;
  },

  parseFromString(string) {
    let doc;

    try {
      const errorHandler = {};

      errorHandler.error = function (m) {
        die(['Error parsing XML file.'].concat((m || '').split(/[\r\n]/)));
      };

      errorHandler.fatalError = errorHandler.error;

      errorHandler.warn = function (m) {
        // ALOY-840: die on unclosed XML tags
        // xmldom hardcodes this as a warning with the string message 'unclosed xml attribute'
        // even when it's a tag that's unclosed
        if (!(m.indexOf('unclosed xml attribute') === -1)) {
          const message = m.replace('unclosed xml attribute', 'Unclosed XML tag or attribute');
          die(['Error parsing XML file.'].concat((message || '').split(/[\r\n]/)));
        }
      };

      errorHandler.warning = errorHandler.warn;
      doc = new _xmldom.DOMParser({
        errorHandler,
        locator: {}
      }).parseFromString(string);
    } catch (e) {
      die('Error parsing XML file.', e);
    }

    return doc;
  },

  parseFromFile(filename) {
    const xml = _fsExtra.default.readFileSync(filename, 'utf8');

    return XML.parseFromString(xml);
  },

  createEmptyNode(name, ns) {
    return XML.parseFromString(`<${name} ${ns}></${name}>`).documentElement;
  },

  toString(node) {
    return new _xmldom.XMLSerializer().serializeToString(node);
  },

  previousSiblingElement(node) {
    if (!node || !node.previousSibling || node.previousSibling === null) {
      return null;
    }

    if (node.previousSibling.nodeType === 1) {
      return node.previousSibling;
    }

    return XML.previousSiblingElement(node.previousSibling);
  }

};
exports.XML = XML;

const getAndValidateProjectPaths = async packageOrRoot => {
  let projectPath = _path.default.resolve(packageOrRoot); // See if we got the "app" path or the project path as an argument


  projectPath = (await _fsExtra.default.exists(_path.default.join(projectPath, '..', CONSTANTS.TIAPP_XML_FILE))) ? _path.default.join(projectPath, '..') : projectPath; // Assign paths objects

  const paths = {
    project: projectPath,
    rootDir: projectPath,
    app: _path.default.join(projectPath, CONSTANTS.ALLOY_DIR),
    indexBase: _path.default.join(CONSTANTS.DIR.CONTROLLER, `${CONSTANTS.NAME_DEFAULT}.${CONSTANTS.FILE_EXT.CONTROLLER}`)
  };
  paths.index = _path.default.join(paths.app, paths.indexBase);
  paths.assets = _path.default.join(paths.app, CONSTANTS.DIR.ASSETS);
  paths.resources = _path.default.join(paths.rootDir, CONSTANTS.RESOURCES_DIR);
  paths.tiappXmlTemplate = _path.default.join(paths.rootDir, CONSTANTS.TIAPP_XML_TEMPLATE_FILE);
  paths.miapprc = _path.default.join(paths.rootDir, CONSTANTS.TIAPP_XML_CONFIGURATION_FILE);
  paths.appPkgPath = _path.default.join(paths.rootDir, 'package.json');
  paths.tiappXml = _path.default.join(paths.rootDir, CONSTANTS.TIAPP_XML_FILE);
  paths.platform = _path.default.join(paths.rootDir, CONSTANTS.DIR.PLATFORM);
  paths.i18n = _path.default.join(paths.rootDir, CONSTANTS.DIR.I18N);
  paths.git = _path.default.join(paths.rootDir, CONSTANTS.DIR.GIT);
  paths.gitIgnore = _path.default.join(paths.rootDir, CONSTANTS.GIT_IGNORE);
  paths.tiModuleConfig = _path.default.join(paths.rootDir, CONSTANTS.TITANIUM_MODULE_CONFIG_FILE);
  paths.skipPlugin = _path.default.join(paths.rootDir, CONSTANTS.SKIP_PLUGIN_FILE); // Validate project configuration paths (tiapp-composer)

  paths.isTemplateExistent = await _fsExtra.default.exists(paths.tiappXmlTemplate); // Check if Titanium Module directory

  paths.isTitaniumModule = await _fsExtra.default.exists(paths.tiModuleConfig); // TODO: https://jira.appcelerator.org/browse/TIMOB-14683
  // Resources/app.js must be present, even if not used

  const appjs = _path.default.join(paths.resources, 'app.js');

  if ((await _fsExtra.default.exists(appjs)) === false) {
    await _fsExtra.default.ensureDir(paths.resources, 755);
    await _fsExtra.default.writeFile(appjs, '');
  }

  paths.settingsPlistFile = _path.default.join(paths.rootDir, CONSTANTS.DIR.SETTINGS_IOS, 'Root.plist');
  paths.settingsAboutPlistFile = _path.default.join(paths.rootDir, CONSTANTS.DIR.SETTINGS_IOS, 'About.plist');
  return paths;
};

exports.getAndValidateProjectPaths = getAndValidateProjectPaths;

function createErrorOutput(msg, e) {
  const errs = [msg || 'An unknown error occurred'];
  const posArray = [];

  if (e) {
    const line = e.line || e.lineNumber;
    if (e.message) errs.push(e.message.split('\n'));
    if (line) posArray.push(`line ${line}`);
    if (e.col) posArray.push(`column ${e.col}`);
    if (e.pos) posArray.push(`position ${e.pos}`);
    if (posArray.length) errs.push(posArray.join(', ')); // add the stack trace if we don't get anything good

    if (errs.length < 2) errs.unshift(e.stack);
  } else {
    errs.unshift(e.stack);
  }

  return errs;
}

function die(msg, e) {
  if (e) {// TODO: Add logger
    // logger.error(createErrorOutput(msg, e));
  } // eslint-disable-next-line no-process-exit


  process.exit(1);
}

function dieWithNode(node, msg) {
  const message = _lodash.default.isArray(msg) ? msg : [msg];
  message.unshift(`Error with <${node.nodeName}> at line ${node.lineNumber}`);
  die(message);
}

const replaceVersionInStringsFile = async (file, version) => {
  const xmlString = await _fsExtra.default.readFile(file, {
    encoding: 'utf-8'
  });
  const doc = XML.parseFromString(xmlString);

  const selectPath = _xpath.default.useNamespaces(CONSTANTS.TI_XPATH_NAMESPACES);

  const node = selectPath(CONSTANTS.STRINGS_VERSION_XPATH, doc, true);

  if (node == null) {
    throw new _errors.NamedError('NodeNotFoundError', `Could not find ${CONSTANTS.STRINGS_VERSION_XPATH} for ${file}`);
  }

  node.textContent = `App Version ${version}`;
  await _fsExtra.default.writeFile(file, XML.toString(doc), {
    encoding: 'utf-8'
  });
  return {
    file,
    changed: node.textContent
  };
};

exports.replaceVersionInStringsFile = replaceVersionInStringsFile;

const trim = line => String(line).replace(/^\s\s*/, '').replace(/\s\s*$/, '');
/**
 * Run fastlane actions as a bundle from given working directory (process.cwd).
 *
 * @param {String} fastlaneAction The fastlane action to run e.g. supply, deliver etc.
 * @param {Array} args Additional args to run within the action
 * @param {Object} options Additional options
 * @param {String} [options.cwd=process.cwd()] The working directory to run the action from
 * @param {Boolean} [options.pipeStdout=false] Pipe stdout instead of printing to the parent process stdout
 * @param {Function} [options.onStdoutDataReceived] A callback to run if piping stdout
 * @param {Object} [options.env={}] Additional env vars to set
 * @returns {Promise<Object>} A promise that resolves with the result from `execa`
 */


exports.trim = trim;

async function runBundleAction(fastlaneAction, args = [], options = {}) {
  const {
    cwd = process.cwd(),
    pipeStdout = false,
    onStdoutDataReceived,
    env = {}
  } = options;
  const fastlaneCmd = (0, _execa.default)('bundle', ['exec', 'fastlane', fastlaneAction, ...args], {
    cwd,
    stdin: 'inherit',
    stdout: pipeStdout === true ? 'pipe' : 'inherit',
    stderr: 'pipe',
    stripFinalNewline: true,
    extendEnv: true,
    env: (0, _extends2.default)({
      FASTLANE_HIDE_LOGIN_INFORMATION: 1,
      FASTLANE_HIDE_CHANGELOG: 1,
      FASTLANE_HIDE_TIMESTAMP: 1,
      // FASTLANE_DISABLE_COLORS: pipeStdout ? 1 : null,
      // FASTLANE_DISABLE_OUTPUT_FORMAT: pipeStdout ? 1 : null,
      // FASTLANE_DISABLE_ANIMATION: 1,
      FASTLANE_SKIP_UPDATE_CHECK: 1
    }, env)
  });

  if (pipeStdout === true && fastlaneCmd.stdout && typeof onStdoutDataReceived === 'function') {
    fastlaneCmd.stdout.on('data', data => onStdoutDataReceived(data.toString()));
  }

  try {
    return await fastlaneCmd;
  } catch (err) {
    throw new Error(trim(err.stderr));
  }
}